#include <iostream>
#include <string.h>
#include <gtest/gtest.h>
#include <libpcsc_cpptools/ScHex.h>
#include <libpcsc_cpptools/ScReader.h>
#include <libpcsc_cpptools/ScBasicCommand.h>

#define TEST_Hex(FUNCTION) do {                                                   \
    FUNCTION("", "", 0);                                                          \
    FUNCTION("42", "\x42", 1);                                                    \
    FUNCTION("42 00 FF AA 24", "\x42\x00\xff\xaa\x24", 5);                        \
    FUNCTION("00 01 02 03 04 05 06 FF", "\x00\x01\x02\x03\x04\x05\x06\xff", 8);   \
    } while (0)

#define TEST_byteArrayToString(STR, DATA, SIZE) \
  ASSERT_EQ(STR, ScHex::byteArrayToString(std::string((char*)memcpy(new BYTE[SIZE], DATA, SIZE), SIZE)))

TEST(Hex, ByteArrayToString)
{
    TEST_Hex(TEST_byteArrayToString);
}

#define TEST_stringToByteArray(STR, DATA, SIZE) do {    \
    auto bytes = ScHex::stringToByteArray(STR);         \
    ASSERT_EQ(SIZE, bytes.size());                   \
    ASSERT_EQ(0, memcmp(bytes.data(), DATA, SIZE));  \
    } while (0)

TEST(Hex, StringToByteArray)
{
    TEST_Hex(TEST_stringToByteArray);
}

#define TEST_command(STR) do {                              \
    auto res = reader->transmit(ScBasicCommand(STR));       \
    ASSERT_NE(nullptr, res->getData().data());              \
    auto resp = ScHex::byteArrayToString(res->getData());   \
    std::cout << "Response (" << res->getData().size()      \
        << "): " << resp << std::endl;                      \
    } while (0)


TEST(Reader, Test1)
{
    std::cout << "Ensure that at lest 1 reader is connected";
    getchar();
    ASSERT_TRUE(ScReader::establishContext());
    auto readers = ScReader::getReaders();
    ASSERT_GE(readers.size(), 1);
    for (auto reader : readers)
    {
        std::cout << reader->getName() << std::endl;
    }
    auto reader = readers[0];
    ASSERT_TRUE(reader->connect());
    std::cout << "ATR: " << ScHex::byteArrayToString(reader->getAtr())
        << " Result: " << reader->getLastResultString() << std::endl;

    TEST_command("94a4 0800 04 2000 2020");
    TEST_command("94b2 0104 1d");
    TEST_command("94d2 0000 1d 5A 00 E0 00 00 50 5C 7C 58 B2 2A DA 8C 20 01 EA D9 80 1E 04 40 20 0E 3F FF FF FF FF FF");

    ASSERT_TRUE(reader->disconnect());
    ASSERT_TRUE(ScReader::freeContext());
}

int main(int argc, char* argv[])
{
  ::testing::InitGoogleTest(&argc, argv);
  return RUN_ALL_TESTS();

  /*std::cout << ScReader::establishContext() << std::endl;
  auto readers = ScReader::getReaders();
  for (auto reader : readers)
  {
    std::cout << reader->getName() << " ";
    std::cout << reader->connect() << " ";
    std::cout << reader->disconnect();
    std::cout << std::endl;
  }
  std::cout << ScReader::freeContext() << std::endl;
  return 0;*/
}
